TxnsCache

Struct TxnsCache 

pub struct TxnsCache<T, C: TxnsCodec = TxnsCodecDefault> { /* private fields */ }
A self-updating TxnsCacheState.

Implementations

Source

impl<T, C> TxnsCache<T, C>

pub async fn open( client: &PersistClient, txns_id: ShardId, only_data_id: Option<ShardId>, ) -> Self

Returns a TxnsCache reading from the given txn shard.

txns_id identifies which shard will be used as the txns WAL. MZ will likely have one of these per env, used by all processes and the same across restarts.

pub async fn update_gt(&mut self, ts: &T) -> &T

Invariant: afterward, self.progress_exclusive will be > ts

Returns the progress_exclusive of the cache after updating.

pub async fn update_ge(&mut self, ts: &T) -> &T

Invariant: afterward, self.progress_exclusive will be >= ts

Returns the progress_exclusive of the cache after updating.

Methods from Deref<Target = TxnsCacheState<T>>

pub fn txns_id(&self) -> ShardId

Returns the ShardId of the txns shard.

pub fn registered_at_progress(&self, data_id: &ShardId, ts: &T) -> bool

Returns whether the data shard was registered to the txns set as of the current progress.

Specifically, a data shard is registered if the most recent register timestamp is set but the most recent forget timestamp is not set.

This function accepts a timestamp as input, but that timestamp must be equal to the progress exclusive, or else the function panics. It mainly acts as a way for the caller to think about the logical time at which this function executes. Times in the past may have been compacted away, and we can’t always return an accurate answer. If this function isn’t sufficient, you can usually find what you’re looking for by inspecting the times in the most recent registration.

pub fn data_snapshot(&self, data_id: ShardId, as_of: T) -> DataSnapshot<T>

Returns a token exchangeable for a snapshot of a data shard.

A data shard might be definite at times past the physical upper because of invariants maintained by this txn system. As a result, this method discovers the latest potentially unapplied write before the as_of.

Callers must first wait for TxnsCache::update_gt with the same or later timestamp to return. Panics otherwise.

pub fn data_listen_next(&self, data_id: &ShardId, ts: &T) -> DataListenNext<T>

Returns the next action to take when iterating a Listen on a data shard.

A data shard Listen is executed by repeatedly calling this method with an exclusive progress frontier. The returned value indicates an action to take. Some of these actions advance the progress frontier, which results in calling this method again with a higher timestamp, and thus a new action. See DataListenNext for specifications of the actions.

Note that this is a state machine on self.progress_exclusive and the listen progress. DataListenNext indicates which state transitions to take.

pub fn min_unapplied_ts(&self) -> &T

Returns the minimum timestamp not known to be applied by this cache.